unit Globals;

interface

type
  TTokenType = (
    tkEof,          tkIdent,        tkNumber,       tkFloat,      tkString,
    tkBegin,        tkEnd,          tkProcedure,    tkVar,        tkConst,
    tkFunction,     tkProgram,      tkType,         tkUses,       tkRecord,
    tkPacked,       tkDiv,          tkMod,          tkUndefined,  tkRange,
    tkLoadfunc,     tkLoadarray,
    tkPeriod,       tkName,         tkAssignment,   tkEqual,      tkMinus,
    tkPlus,         tkMultiply,     tkSemicolon,    tkColon,      tkComma,
    tkOpenParen,    tkCloseParen,   tkOpenAParen,   tkCloseAParen,tkDivide);

const
  TokenTypeText: array[TTokenType] of String = (
    'END OF FILE',  'IDENTIFIER',   'NUMBER',       'FLOAT',      'STRING',
    'BEGIN',        'END',          'PROCEDURE',    'VAR',        'CONST',
    'FUNCTION',     'PROGRAM',      'TYPE',         'USES',       'RECORD',
    'PACKED',       'DIV',          'MOD',          'UNDEFINED',  '..',
    'LOADFUNC',     'LOADARRAY',
    '.',            '',             ':=',           '=',          '-',
    '+',            '*',            ';',            ':',          ',',
    '(',            ')',            '[',            ']',          '/');

type
  PType       = record
    int,low   : integer;
    float     : double;
    str       : string;
    null      : byte;
    tok       : TTokenType;
  end;

var
  Token       : TTokenType;
  NextTok     : Char;
  CurrTok     : Char;
  Data,P,D    : PType;
  CP          : Integer;
  StackArr    : array of PType;

  function  CharToOem(s: String;const f: Boolean = true): String;
  procedure EmitLn(D: String);

implementation

// #######################   ########################

function  CharToOem(s: String;const f: Boolean = true): String;
var
  i,code: Integer;
  ret   : String;
begin
  ret:='';
  for i:=1 to length(s) do
   begin
     code:=ord(s[i]);
     if f then
      case code of
        192..239: code:=code-64;
        240..255: code:=code-16;
      end else
      case code of
        128..175: code:=code+64;
        224..239: code:=code+16;
      end;
     Ret:=Ret+chr(code);
   end;
  Result:=Ret;
end;

procedure EmitLn(D: String);
begin
  writeln(CharToOem(D));
end;

//-------------------

end.
