unit Scaner;

interface

uses
  Globals;

  procedure GetChar;
  procedure NextToken;

implementation

// ##############################  #################################

procedure GetChar;
begin
  CurrTok := NextTok;
  Read(NextTok);
end;

procedure NextToken;
label
  Redo;
var
  IsCompilerDirective,Flag  : Boolean;
  Directive                 : String;
  i                         : integer;

  function SameText(const S1, S2: string): Boolean;
  var
    i,n: integer;
  begin
    if length(S1)<>length(S2) then
    begin
      Result:=false;
      Exit;
    end;
    n:=0;
    for i:=1 to length(S1) do
      if UpCase(S1[i])=UpCase(S2[i]) then n:=n+1;
    if n=length(S1) then Result:=true else Result:=false;
  end;

  function TrimRight(const S: string): string;
  var
    I: Integer;
  begin
    I := Length(S);
    while (I > 0) and (S[I] <= ' ') do Dec(I);
    Result := Copy(S, 1, I);
  end;

  procedure DirectiveSpace;
  begin
    if IsCompilerDirective and
       ((Directive = '') or (Directive[Length(Directive)] <> ' ')) then
      Directive := Directive + ' ';
  end;

  procedure HandleCompilerDirective;
  begin
    if not IsCompilerDirective then Exit;
    Directive := TrimRight(Directive);
    if SameText(Directive, 'APPTYPE CONSOLE') then
      writeln(' - {$APPTYPE CONSOLE}')
    else if SameText(Directive, 'APPTYPE GUI') then
      writeln(' - {$APPTYPE GUI}')
    else if SameText(Copy(Directive, 1, 2), 'I ') then
      writeln(' - {$I}')//HandleInclude (Copy(Directive, 3, Maxint))
    else
      EmitLn ('  ');//'Invalid compiler directive: ''%s''', [Directive]);
  end;

begin

Token     := tkIdent;
Data.Str  := '';
Flag      := false;

Redo:
  repeat
    GetChar;
  until not(CurrTok in [#1..#32]);

  case CurrTok of

    ':':
              begin
                if NextTok = '=' then begin
                  Token := tkAssignment;
                  GetChar;
                end
                else begin
                  Token := tkColon;
                end;
              end;

    '.':
              begin
                if NextTok = '.' then begin
                  Token := tkRange;
                  GetChar;
                end
                else
                  Token := tkPeriod;
              end;
    '(':
              begin
                if NextTok = '*' then begin  { handle (* *)-style comments }
                  GetChar;
                  Directive := '';
                  IsCompilerDirective := NextTok = '$';
                  if IsCompilerDirective then GetChar;
                  while True do begin
                    GetChar;
                    case CurrTok of
                      #0      : EmitLn (' ');
                      #1..' ' : DirectiveSpace;
                      '*'     : if NextTok = ')' then
                                begin
                                  GetChar;
                                  Break;
                                end;
                    else
                      Directive := Directive + CurrTok;
                    end;
                  end;
                  HandleCompilerDirective;
                  goto Redo;
                end;
                Token := tkOpenParen;
              end;

    '{':
              begin
                Directive := '';
                IsCompilerDirective := NextTok = '$';
                if IsCompilerDirective then GetChar;
                while True do begin
                  GetChar;
                  case CurrTok of
                    #0:  Emitln (' ');
                    #1..' ': DirectiveSpace;
                    '}': Break;
                  else
                    Directive := Directive + CurrTok;
                  end;
                end;
                HandleCompilerDirective;
                goto Redo;
              end;

    '=':      Token := tkEqual;
    '+':      Token := tkPlus;
    '-':      Token := tkMinus;
    '*':      Token := tkMultiply;
    ',':      Token := tkComma;
    ';':      Token := tkSemicolon;
    ')':      Token := tkCloseParen;
    '[':      Token := tkOpenAParen;
    ']':      Token := tkCloseAParen;

    '/':
              begin
                if NextTok <> '/' then Token := tkDivide
                else begin
                  repeat GetChar; until CurrTok in [#0, #10];
                  goto Redo;
                end;
              end;

    '0'..'9':
              begin
                Token := tkNumber;
                Data.int := 0;
                Data.float := 0;
                Data.low := 0;
                i:=10;
                while True do begin
                  Data.str := Data.str + CurrTok;
                  if flag then
                  begin
                    Data.float := Data.float + (Ord(CurrTok) - Ord('0'))/i;
                    Data.low := Data.low * 10 + (Ord(CurrTok) - Ord('0'));
                    i:=i*10;
                  end else
                    Data.int := Data.int * 10 + (Ord(CurrTok) - Ord('0'));
                  if not(NextTok in ['0'..'9','.']) then Break;
                  GetChar;
                  if (CurrTok in ['.']) then
                  begin
                    flag:=true;
                    Data.str := Data.str + '.';
                    Data.float := Data.int;
                    GetChar;
                  end;
                end;
                if flag then Token := tkFloat;
              end;
    '$':
              begin
                Token := tkNumber;
                while True do begin
                  Data.str := Data.str + CurrTok;
                  if not(NextTok in ['0'..'9', 'A'..'F', 'a'..'f']) then Break;
                  GetChar;
                end;
                if Data.str = '$' then
                  Emitln ('  ');
              end;

    '''', '#':
              begin
                Token := tkString;
                while True do begin
                  case CurrTok of
                    '''':
                        begin
                          GetChar;
                          while True do
                          begin
                            case CurrTok of
                              #0, #10, #13: Emitln (' a');
                              '''':
                                begin
                                  GetChar;
                                  if CurrTok <> '''' then Break;
                                end;
                            end;
                            Data.str := Data.str + CurrTok;
                            GetChar;
                          end;
                        end;
                    '#':
                        begin
                          GetChar;
                          Data.int := 0;
                          while CurrTok in ['0'..'9'] do
                          begin
                            Data.int := Data.int * 10 + (Ord(CurrTok) - Ord('0'));
                            GetChar;
                          end;
                          if (Data.int > 127) then writeln ('Char constant out of range');
                          Data.str := Data.str + Chr(Data.int);
                        end;
                  else
                    Break;
                  end;
                end;
              end;

    'A'..'Z',
    'a'..'z':
              begin
                while True do begin
                  Data.str := Data.str + CurrTok;
                  if NextTok = '(' then Token := tkLoadfunc;
                  if NextTok = '[' then Token := tkLoadarray;
                  if not(NextTok in ['A'..'Z', 'a'..'z', '0'..'9', '_']) then Break;
                  GetChar;
                end;
                if      SameText(Data.str, 'begin')       then
                  Token := tkBegin
                else if SameText(Data.str, 'end')         then
                  Token := tkEnd
                else if SameText(Data.str, 'procedure')   then
                  Token := tkProcedure
                else if SameText(Data.str, 'function')    then
                  Token := tkFunction
                else if SameText(Data.str, 'type')        then
                  Token := tkType
                else if SameText(Data.str, 'uses')        then
                  Token := tkUses
                else if SameText(Data.str, 'var')         then
                  Token := tkVar
                else if SameText(Data.str, 'const')       then
                  Token := tkConst
                else if SameText(Data.str, 'program')     then
                  Token := tkProgram
                else if SameText(Data.str, 'record')      then
                  Token := tkRecord
                else if SameText(Data.str, 'packed')      then
                  Token := tkPacked
                else if SameText(Data.str, 'div')         then
                  Token := tkDiv
                else if SameText(Data.str, 'mod')         then
                  Token := tkMod
                else if not (Token in [tkLoadfunc,tkLoadarray]) then
                  Token := tkIdent;
              end;
  else
    Token := tkUndefined;
    if NextTok = ^Z then Token := tkEof;
  end;
  Data.tok := Token;
  if Data.str='' then Data.str := CurrTok;
end;

//#########################################################################

end.
